@@ -1,5 +1,6 @@
 /* ######################### includes ############################################################################### */
 
+#include <algorithm>
 #include <cmath>
 #include <cfloat>
 #include <climits>
@@ -360,7 +361,7 @@ err_t tooltip_list_entry_delete(tooltip_list_entry_t entry)
 static void moveTransformationHelper(const std::shared_ptr<GRM::Element> &element, double ndc_x, double ndc_y,
                                      double xshift, double yshift, bool is_movable)
 {
-  double tmp = 0, x_with_shift, y_with_shift, x_ndc, y_ndc;
+  double x_with_shift, y_with_shift, x_ndc, y_ndc;
   double old_x_shift = 0, old_y_shift = 0, wc_x_shift, wc_y_shift;
   int width, height, max_width_height;
   std::string post_fix = "_wc";
@@ -370,13 +371,13 @@ static void moveTransformationHelper(const std::shared_ptr<GRM::Element> &elemen
                                                        "labels_group",
                                                        "titles_3d",
                                                        "text",
-                                                       "y_tick_label_group",
-                                                       "x_tick_label_group",
                                                        "layout_grid_element",
                                                        "layout_grid",
                                                        "central_region",
                                                        "side_region",
-                                                       "marginal_heatmap_plot"};
+                                                       "marginal_heatmap_plot",
+                                                       "legend",
+                                                       "axis"};
 
   GRM::Render::getFigureSize(&width, &height, nullptr, nullptr);
   max_width_height = grm_max(width, height);
@@ -577,7 +578,7 @@ int grm_input(const grm_args_t *input_args)
 
               if (static_cast<int>(subplot_element->getAttribute("x_log")))
                 {
-                  for (int j = 0; j < x_length; j++)
+                  for (int j = 0; j < (int)x_length; j++)
                     {
                       if (!grm_isnan(x_series_vec[j]))
                         {
@@ -590,7 +591,7 @@ int grm_input(const grm_args_t *input_args)
                 }
               if (static_cast<int>(subplot_element->getAttribute("y_log")))
                 {
-                  for (int j = 0; j < y_length; j++)
+                  for (int j = 0; j < (int)y_length; j++)
                     {
                       if (!grm_isnan(y_series_vec[j]))
                         {
@@ -636,7 +637,7 @@ int grm_input(const grm_args_t *input_args)
 
               if (static_cast<int>(subplot_element->getAttribute("x_log")))
                 {
-                  for (int j = 0; j < x_length - x_offset; j++)
+                  for (int j = 0; j < (int)x_length - x_offset; j++)
                     {
                       double step_x = (x_series_vec[x_length - 1] - x_min) / (x_length - x_offset);
                       double tmp = 0, a = x_min + j * step_x, b = x_min + (j + 1) * step_x;
@@ -650,7 +651,7 @@ int grm_input(const grm_args_t *input_args)
                 }
               if (static_cast<int>(subplot_element->getAttribute("y_log")))
                 {
-                  for (int j = 0; j < y_length - y_offset; j++)
+                  for (int j = 0; j < (int)y_length - y_offset; j++)
                     {
                       double step_y = (y_series_vec[y_length - 1] - y_min) / (y_length - y_offset);
                       double tmp = 0, a = y_min + j * step_y, b = y_min + (j + 1) * step_y;
@@ -668,7 +669,7 @@ int grm_input(const grm_args_t *input_args)
                 {
                   double tmp = 0;
                   xind_d = 0;
-                  for (int j = 0; j < x_length - x_offset; j++)
+                  for (int j = 0; j < (int)x_length - x_offset; j++)
                     {
                       if (tmp + x_steps[j] > (x - x_0)) break;
                       tmp += x_steps[j];
@@ -681,7 +682,7 @@ int grm_input(const grm_args_t *input_args)
                 {
                   double tmp = 0;
                   yind_d = 0;
-                  for (int j = 0; j < y_length - y_offset; j++)
+                  for (int j = 0; j < (int)y_length - y_offset; j++)
                     {
                       if (tmp + y_steps[j] < (y - y_0)) break;
                       tmp += y_steps[j];
@@ -718,9 +719,6 @@ int grm_input(const grm_args_t *input_args)
                           if (childKind == "hist")
                             {
                               // reset bar colors
-                              bool is_horizontal =
-                                  static_cast<std::string>(child->getAttribute("orientation")) == "horizontal";
-
                               // bar level
                               for (auto &bars : child->children())
                                 {
@@ -739,8 +737,6 @@ int grm_input(const grm_args_t *input_args)
                                           break;
                                         }
 
-                                      int fillColorInd =
-                                          static_cast<int>(innerFillGroup->getAttribute("fill_color_ind"));
                                       if (xind != -1)
                                         {
                                           innerFillGroup->children()[xind]->removeAttribute("fill_color_ind");
@@ -990,7 +986,6 @@ int grm_is3d(const int x, const int y)
 {
   int width, height, max_width_height;
   double ndc_x, ndc_y;
-  const char *kind;
 
   GRM::Render::getFigureSize(&width, &height, nullptr, nullptr);
   max_width_height = grm_max(width, height);
@@ -1234,7 +1229,7 @@ error_cleanup:
 
 err_t get_tooltips(int mouse_x, int mouse_y, err_t (*tooltip_callback)(int, int, grm_tooltip_info_t *))
 {
-  double *x_series, *y_series, *z_series, x, y, x_min, x_max, y_min, y_max, mindiff = DBL_MAX, diff;
+  double x, y, x_min, x_max, y_min, y_max, mindiff = DBL_MAX, diff;
   double x_range_min, x_range_max, y_range_min, y_range_max, x_px, y_px;
   int width, height, max_width_height;
   std::vector<std::string> labels;
@@ -1283,12 +1278,15 @@ err_t get_tooltips(int mouse_x, int mouse_y, err_t (*tooltip_callback)(int, int,
   gr_savestate();
   auto central_region = subplot_element->querySelectors("central_region");
   GRM::Render::processWindow(central_region);
-  if (central_region->hasAttribute("window_x_min"))
+  if (central_region->hasAttribute("viewport_x_min"))
     {
       gr_setviewport(static_cast<double>(central_region->getAttribute("viewport_x_min")),
                      static_cast<double>(central_region->getAttribute("viewport_x_max")),
                      static_cast<double>(central_region->getAttribute("viewport_y_min")),
                      static_cast<double>(central_region->getAttribute("viewport_y_max")));
+    }
+  if (central_region->hasAttribute("window_x_min") && kind != "pie")
+    {
       gr_setwindow(static_cast<double>(central_region->getAttribute("window_x_min")),
                    static_cast<double>(central_region->getAttribute("window_x_max")),
                    static_cast<double>(central_region->getAttribute("window_y_min")),
@@ -1300,36 +1298,33 @@ err_t get_tooltips(int mouse_x, int mouse_y, err_t (*tooltip_callback)(int, int,
 
   auto axes_vec = subplot_element->querySelectorsAll("axes");
   auto coordinate_system = subplot_element->querySelectors("coordinate_system");
-  std::vector<std::shared_ptr<GRM::Element>> label_vec;
+  auto left_side_region = subplot_element->querySelectors("side_region[location=\"left\"]");
+  auto bottom_side_region = subplot_element->querySelectors("side_region[location=\"bottom\"]");
+  struct
+  {
+    std::shared_ptr<GRM::Element> x, y;
+  } label_elements{};
 
-  if (coordinate_system && coordinate_system->hasAttribute("x_label") && coordinate_system->hasAttribute("y_label"))
-    label_vec.push_back(coordinate_system);
+  if (bottom_side_region && bottom_side_region->hasAttribute("text_content")) label_elements.x = bottom_side_region;
+  if (left_side_region && left_side_region->hasAttribute("text_content")) label_elements.y = left_side_region;
 
-  if (label_vec.empty())
+  if (label_elements.x && label_elements.x->hasAttribute("text_content"))
     {
-      info->xlabel = (char *)"x";
-      info->ylabel = (char *)"y";
+      static auto xlabel = static_cast<std::string>(label_elements.x->getAttribute("text_content"));
+      info->xlabel = (char *)xlabel.c_str();
     }
   else
     {
-      if (!label_vec[0]->hasAttribute("x_label"))
-        {
-          info->xlabel = (char *)"x";
-        }
-      else
-        {
-          static std::string xlabel = static_cast<std::string>(label_vec[0]->getAttribute("x_label"));
-          info->xlabel = (char *)xlabel.c_str();
-        }
-      if (!label_vec[0]->hasAttribute("y_label"))
-        {
-          info->ylabel = (char *)"y";
-        }
-      else
-        {
-          static std::string ylabel = static_cast<std::string>(label_vec[0]->getAttribute("y_label"));
-          info->ylabel = (char *)ylabel.c_str();
-        }
+      info->xlabel = (char *)"x";
+    }
+  if (label_elements.y && label_elements.y->hasAttribute("text_content"))
+    {
+      static auto ylabel = static_cast<std::string>(label_elements.y->getAttribute("text_content"));
+      info->ylabel = (char *)ylabel.c_str();
+    }
+  else
+    {
+      info->ylabel = (char *)"y";
     }
 
   x_range_min = (double)(mouse_x - 50) / max_width_height;
@@ -1544,7 +1539,7 @@ err_t get_tooltips(int mouse_x, int mouse_y, err_t (*tooltip_callback)(int, int,
 
               if (static_cast<int>(subplot_element->getAttribute("x_log")))
                 {
-                  for (int j = 0; j < x_length; j++)
+                  for (int j = 0; j < (int)x_length; j++)
                     {
                       if (!grm_isnan(x_series_vec[j]))
                         {
@@ -1557,7 +1552,7 @@ err_t get_tooltips(int mouse_x, int mouse_y, err_t (*tooltip_callback)(int, int,
                 }
               if (static_cast<int>(subplot_element->getAttribute("y_log")))
                 {
-                  for (int j = 0; j < y_length; j++)
+                  for (int j = 0; j < (int)y_length; j++)
                     {
                       if (!grm_isnan(y_series_vec[j]))
                         {
@@ -1582,7 +1577,7 @@ err_t get_tooltips(int mouse_x, int mouse_y, err_t (*tooltip_callback)(int, int,
 
               if (static_cast<int>(subplot_element->getAttribute("x_log")))
                 {
-                  for (int j = 0; j < x_length - x_offset; j++)
+                  for (int j = 0; j < (int)x_length - x_offset; j++)
                     {
                       double step_x = (x_series_vec[x_length - 1] - x_min) / (x_length - x_offset);
                       double tmp = 0, a = x_min + j * step_x, b = x_min + (j + 1) * step_x;
@@ -1596,7 +1591,7 @@ err_t get_tooltips(int mouse_x, int mouse_y, err_t (*tooltip_callback)(int, int,
                 }
               if (static_cast<int>(subplot_element->getAttribute("y_log")))
                 {
-                  for (int j = 0; j < y_length - y_offset; j++)
+                  for (int j = 0; j < (int)y_length - y_offset; j++)
                     {
                       double step_y = (y_series_vec[y_length - 1] - y_min) / (y_length - y_offset);
                       double tmp = 0, a = y_min + j * step_y, b = y_min + (j + 1) * step_y;
@@ -1627,7 +1622,7 @@ err_t get_tooltips(int mouse_x, int mouse_y, err_t (*tooltip_callback)(int, int,
                 {
                   double tmp = 0;
                   x_series_idx = x_offset;
-                  for (int j = 0; j < x_length - x_offset; j++)
+                  for (int j = 0; j < (int)x_length - x_offset; j++)
                     {
                       if (tmp + x_steps[j] > (mouse_x - x_0)) break;
                       tmp += x_steps[j];
@@ -1641,7 +1636,7 @@ err_t get_tooltips(int mouse_x, int mouse_y, err_t (*tooltip_callback)(int, int,
                 {
                   double tmp = 0;
                   y_series_idx = y_offset;
-                  for (int j = 0; j < y_length - y_offset; j++)
+                  for (int j = 0; j < (int)y_length - y_offset; j++)
                     {
                       if (tmp + y_steps[j] < (mouse_y - y_0)) break;
                       tmp += y_steps[j];
@@ -1690,31 +1685,23 @@ err_t get_tooltips(int mouse_x, int mouse_y, err_t (*tooltip_callback)(int, int,
               info->y_px = -1;
               info->x = 0;
               info->y = 0;
-              if (label_vec.empty())
+              if (label_elements.x && label_elements.x->hasAttribute("text_content"))
                 {
-                  info->xlabel = (char *)"x";
-                  info->ylabel = (char *)"y";
+                  static auto xlabel = static_cast<std::string>(label_elements.x->getAttribute("text_content"));
+                  info->xlabel = (char *)xlabel.c_str();
                 }
               else
                 {
-                  if (!label_vec[0]->hasAttribute("x_label"))
-                    {
-                      info->xlabel = (char *)"x";
-                    }
-                  else
-                    {
-                      static std::string xlabel = static_cast<std::string>(label_vec[0]->getAttribute("x_label"));
-                      info->xlabel = (char *)xlabel.c_str();
-                    }
-                  if (!label_vec[0]->hasAttribute("y_label"))
-                    {
-                      info->ylabel = (char *)"y";
-                    }
-                  else
-                    {
-                      static std::string ylabel = static_cast<std::string>(label_vec[0]->getAttribute("y_label"));
-                      info->ylabel = (char *)ylabel.c_str();
-                    }
+                  info->xlabel = (char *)"x";
+                }
+              if (label_elements.y && label_elements.y->hasAttribute("text_content"))
+                {
+                  static auto ylabel = static_cast<std::string>(label_elements.y->getAttribute("text_content"));
+                  info->ylabel = (char *)ylabel.c_str();
+                }
+              else
+                {
+                  info->ylabel = (char *)"y";
                 }
               info->label = (char *)"";
               return_error_if(info == nullptr, ERROR_MALLOC);
