@@ -15,6 +15,9 @@ extern "C" {
 #ifdef __cplusplus
 }
 
+#include <optional>
+
+#include <grm/dom_render/context.hxx>
 #include <grm/dom_render/graphics_tree/Element.hxx>
 
 #include <grm/plot.h>
@@ -58,16 +61,27 @@ extern const char *plot_clear_exclude_keys[];
 #define PLOT_DEFAULT_XGRID 1
 #define PLOT_DEFAULT_YGRID 1
 #define PLOT_DEFAULT_ZGRID 1
-#define PLOT_DEFAULT_COLORBAR_MAX_CHAR_HEIGHT 0.016 * DEFAULT_ASPECT_RATIO_FOR_SCALING
-#define PLOT_DEFAULT_COLORBAR_OFFSET 0.02 * DEFAULT_ASPECT_RATIO_FOR_SCALING
-#define PLOT_3D_COLORBAR_OFFSET 0.05 * DEFAULT_ASPECT_RATIO_FOR_SCALING
-#define PLOT_POLAR_COLORBAR_OFFSET 0.025 * DEFAULT_ASPECT_RATIO_FOR_SCALING
+#define PLOT_DEFAULT_COLORBAR_MAX_CHAR_HEIGHT 0.016
+#define PLOT_DEFAULT_COLORBAR_OFFSET 0.02
+#define PLOT_3D_COLORBAR_OFFSET 0.05
+#define PLOT_POLAR_COLORBAR_OFFSET 0.025
 #define PLOT_DEFAULT_COLORBAR_LOCATION "right"
-#define PLOT_DEFAULT_COLORBAR_WIDTH 0.04
-#define DEFAULT_ASPECT_RATIO_FOR_SCALING 4.0 / 3.0
+#define PLOT_DEFAULT_COLORBAR_WIDTH 0.03
 
 /* ========================= datatypes ============================================================================== */
 
+/* ------------------------- dump ----------------------------------------------------------------------------------- */
+
+typedef enum
+{
+  DUMP_AUTO_DETECT = 0,
+  DUMP_JSON_PLAIN = 1,
+  DUMP_JSON_ESCAPE_DOUBLE_MINUS = 2,
+  DUMP_JSON_BASE64 = 3,
+  DUMP_BSON_BASE64 = 4,
+} dump_encoding_t;
+
+
 /* ------------------------- plot ----------------------------------------------------------------------------------- */
 
 /* ~~~~~~~~~~~~~~~~~~~~~~~~~ kind to func ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ */
@@ -166,22 +180,40 @@ int get_free_id_from_figure_elements();
 
 #ifdef __cplusplus
 }
-
 #endif
 
+
+/* ------------------------- dump ----------------------------------------------------------------------------------- */
+
+#ifdef __cplusplus
+void dump_context(FILE *f, dump_encoding_t dump_encoding,
+                  const std::unordered_set<std::string> *context_keys_to_discard = nullptr);
+char *dump_context_str(dump_encoding_t dump_encoding,
+                       const std::unordered_set<std::string> *context_keys_to_discard = nullptr);
+
+void dump_context_as_xml_comment(FILE *f, const std::unordered_set<std::string> *context_keys_to_discard = nullptr);
+char *dump_context_as_xml_comment_str(const std::unordered_set<std::string> *context_keys_to_discard = nullptr);
+#endif
+
+/* ------------------------- load ----------------------------------------------------------------------------------- */
+
+#ifdef __cplusplus
+void load_context_str(GRM::Context &context, const std::string &context_str, dump_encoding_t dump_encoding);
+#endif
+
+
 /* ------------------------- xml ------------------------------------------------------------------------------------ */
 
 #ifdef __cplusplus
-extern "C" {
+#ifndef NO_XERCES_C
+err_t validate_graphics_tree(bool include_private_attributes = false);
+#endif
+bool validate_graphics_tree_with_error_messages();
 #endif
 
-#ifndef NO_LIBXML2
-err_t validate_graphics_tree_xml(void);
-#endif
-int validate_graphics_tree_with_error_messages(void);
 
-#ifdef __cplusplus
-}
+#if defined(__cplusplus) && !defined(NO_XERCES_C)
+std::string get_merged_schema_filepath();
 #endif
 
 #endif /* ifndef GRM_PLOT_INT_H_INCLUDED */
